#include "assert.h"

#include "string.h"
#include "math.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"

#include "BList.h"

#include "auxiliar.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "interfaces.h"
#include "tiles.h"
#include "samples.h"
#include "variables.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activeMAP.h"
#include "activemog2.h"
#include "activeinterface.h"
#include "actionscript.h"
#include "ActiveMoG2Application.h"

#include "randomc.h"

extern int SCREEN_X;
extern int SCREEN_Y;
extern int COLOUR_DEPTH;
extern bool SOUND_ENABLED;
extern bool fullscreen;
extern char *application_name;

/* Redrawing constant: */ 
extern SDL_Surface *screen_sfc;

extern int buffer_fps,act_fps,fps_timer;
extern int buffer_fps_d,fps_d;

extern int n_collisions;
extern int acum_collisions[100];


ActiveMOG2Application::ActiveMOG2Application(MOG2 *game)
{
	int i;

	state=0;
	screen_x=640;
	screen_y=480;

	application_variables.Delete();
	application_variables.Copy(game->application_variables);	
	{
		List<VARIABLE> l,todelete;
		VARIABLE *v;

		l.Instance(application_variables);
		l.Rewind();
		while(l.Iterate(v)) {
			if (v->cmp("screen_x")) {
				v->get(&screen_x,0,0);
				todelete.Add(v);
			} /* if */ 
			if (v->cmp("screen_y")) {
				v->get(&screen_y,0,0);
				todelete.Add(v);
			} /* if */ 
			if (v->cmp("fps")) {
				v->get(&fps,0,0);
				todelete.Add(v);
			} /* if */ 
			if (v->cmp("hw_render")) {
				v->get(&hw_render,0,0);
				todelete.Add(v);
			} /* if */ 
			if (v->cmp("player_plane")) {
				v->get(&player_plane,0,0);
				todelete.Add(v);
			} /* if */ 
			if (v->cmp("sfx_atenuation_min")) {
				v->get(&sfx_atenuation_min,0,0);
				todelete.Add(v);
			} /* if */ 
			if (v->cmp("sfx_atenuation_max")) {
				v->get(&sfx_atenuation_max,0,0);
				todelete.Add(v);
			} /* if */ 
			if (v->cmp("state")) {
				v->get(&state,0,0);
				todelete.Add(v);
			} /* if */ 
		} /* while */ 

		while(!todelete.EmptyP()) {
			v=todelete.ExtractIni();
			application_variables.DeleteElement(v);
			delete v;
		} /* while */ 
	}

	n_states=game->n_app_states;
	s_action=new ActionSCRIPT *[n_states];
	for(i=0;i<n_states;i++) {
		s_action[i]=new ActionSCRIPT(*(game->s_app_action[i]));
		s_action[i]->init();
	} /* for */ 
	application_foreign=0;

	active_game=new ActiveMOG2(game);
	active_game->app=this;
	terminated=false;

	current_cycle=0;

	random_generator=new TRanrotBGenerator(0);

	music_priority=0;
	music_playing=0;
	music_volume=100;
	music_sequence=0;
	music_sequence_ip=0;
	music_part=0;

	sfx_volume=100;

	game_mode=0;
	replay_fp=0;
	keyboardstate=0;
	n_LVKs=0;
	event_time=-1;
	event_key=0;
	event_val=0;

	switch(game_mode) {
	case 0:
		{
			keyboardstate=new KEYBOARDSTATE *[1];
			keyboardstate[0]=new KEYBOARDSTATE();
			n_LVKs=1;
		}
		break;
	case 1:
		break;
	case 2:
		break;
	} /* switch */ 

} /* ActiveMOG2Application::ActiveMOG2Application */ 



ActiveMOG2Application::~ActiveMOG2Application()
{
	int i;

	for(i=0;i<n_states;i++) {
		delete s_action[i];
		s_action[i]=0;
	} /* for */ 
	delete []s_action;
	s_action=0;
	application_foreign=0;

	delete active_game;
	active_game=0;

	if (replay_fp!=0) fclose(replay_fp);
	replay_fp=0;
	{
		int i;
		for(i=0;i<n_LVKs;i++) {
			delete keyboardstate[i];
			keyboardstate[i]=0;
		} /* for */ 
		delete []keyboardstate;
		keyboardstate=0;
	}

	delete random_generator;

	delete []music_playing;
	music_playing=0;
	delete []music_sequence;
	music_sequence=0;


} /* ActiveMOG2Application::ActiveMOG2Application */ 



int ActiveMOG2Application::get_variable_value(char *vname,int *ival,char **sval,ActiveOBJECT **oval)
{
	List<VARIABLE> l;
	VARIABLE *v;

	if (strcmp(vname,"screen_x")==0) {
		*ival=screen_x;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"screen_y")==0) {
		*ival=screen_y;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"music_volume")==0) {
		*ival=music_volume;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"sfx_volume")==0) {
		*ival=sfx_volume;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"music_priority")==0) {
		*ival=music_priority;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"music_playing")==0) {
		*sval=music_playing;
		return VALUE_STRING;
	} else if (strcmp(vname,"state")==0) {
		*ival=state;
		return VALUE_INTEGER;
	} /* if */ 

	l.Instance(application_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) return v->get(ival,sval,oval);
	} /* while */ 
	return VALUE_NONE;
} /* ActiveMOG2Application::get_variable_value */ 


void ActiveMOG2Application::set_variable_value(char *vname,int value)
{
	List<VARIABLE> l;
	VARIABLE *v;


	if (strcmp(vname,"music_volume")==0) {
		music_volume=value;
	} else if (strcmp(vname,"sfx_volume")==0) {
		sfx_volume=value;
	} else if (strcmp(vname,"music_priority")==0) {
		music_priority=value;
	} else if (strcmp(vname,"screen_x")==0) {
		screen_x=value;
	} else if (strcmp(vname,"screen_y")==0) {
		screen_y=value;
	} else if (strcmp(vname,"state")==0) {
		if (value>=0 && value<n_states) {
			state=value;
			if (s_action[state]!=0) s_action[state]->init();
		} /* if */ 	
	} else {
		l.Instance(application_variables);
		l.Rewind();
		while(l.Iterate(v)) {
			if (v->cmp(vname)) {
				v->set(value);
				return;
			} /* if */ 
		} /* while */ 
	} /* if */ 

} /* ActiveMOG2Application::set_variable_value */ 


void ActiveMOG2Application::set_variable_value(char *vname,char *value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(application_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 
} /* ActiveMOG2Application::set_variable_value */ 


void ActiveMOG2Application::set_variable_value(char *vname,ActiveOBJECT *value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(application_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 
} /* ActiveMOG2Application::set_variable_value */ 


void ActiveMOG2Application::Execute(void)
{
	int time,act_time;
	SDL_Event event;
	bool quit=false;
	int extra_flags=0;
	int GAME_REDRAWING_PERIOD=40;

	if (hw_render==0) {
		/* SOFTWARE RENDERING */ 
		extra_flags=0;
	} else {
		/* HARDWARE RENDERING */ 
		extra_flags=SDL_HWSURFACE|SDL_DOUBLEBUF;
	} /* if */ 

	GAME_REDRAWING_PERIOD=1000/fps;
	if (GAME_REDRAWING_PERIOD<0) GAME_REDRAWING_PERIOD=0;
	if (GAME_REDRAWING_PERIOD>1000) GAME_REDRAWING_PERIOD=1000;

//	SDL_QuitSubSystem(SDL_INIT_VIDEO);
//	SDL_InitSubSystem(SDL_INIT_VIDEO);
	screen_sfc = SDL_SetVideoMode(screen_x, screen_y, COLOUR_DEPTH, (fullscreen ? SDL_FULLSCREEN : 0)|extra_flags);
	SDL_WM_SetCaption(application_name, 0);
	SDL_ShowCursor(SDL_DISABLE);
	SDL_EnableUNICODE(1);

	if (SOUND_ENABLED) {
		init_music(this);
		Mix_HookMusic(myMusicPlayer, (void *)this);
	} /* if */ 
	
	time=SDL_GetTicks();
	while (!quit) {
		while( SDL_PollEvent( &event ) ) {
			bool captured=false;

            switch( event.type ) {
                /* Keyboard event */ 
                case SDL_KEYDOWN:
					if (event.key.keysym.sym==SDLK_F12) {
						captured=true;
						quit = true;
					} /* if */ 

					if (event.key.keysym.sym==SDLK_RETURN) {
						SDLMod modifiers=SDL_GetModState();

						if ((modifiers&KMOD_ALT)!=0) {
							captured=true;

							/* Toogle FULLSCREEN mode: */ 
							if (fullscreen) fullscreen=false;
									   else fullscreen=true;
//							SDL_QuitSubSystem(SDL_INIT_VIDEO);
//							SDL_InitSubSystem(SDL_INIT_VIDEO);
//							if (SDL_WasInit(SDL_INIT_VIDEO)) {
								screen_sfc = SDL_SetVideoMode(screen_x, screen_y, COLOUR_DEPTH, (fullscreen ? SDL_FULLSCREEN : 0)|extra_flags);
								SDL_WM_SetCaption(application_name, 0);
								SDL_ShowCursor(SDL_DISABLE);
								SDL_EnableUNICODE(1);
//							} else {
//								quit = true;
//							} /* if */ 
						} /* if */ 
					} /* if */ 
                    break;

                /* SDL_QUIT event (window close) */ 
                case SDL_QUIT:
					captured=true;

                    quit = true;
                    break;
            } /* switch */ 

			if (!captured) {
				if (!this->event(&event)) quit=true;
			} /* if */ 
        } /* while */ 

		act_time=SDL_GetTicks();
		while (act_time-time>=GAME_REDRAWING_PERIOD)
		{
			int i;

			for(i=0;i<99;i++) acum_collisions[i]=acum_collisions[i+1];
			acum_collisions[99]=n_collisions;
			n_collisions=0;
			act_fps++;
			if ((act_time-fps_timer)>1000) {
				fps_timer=act_time;
				buffer_fps=act_fps;
				buffer_fps_d=fps_d;
				act_fps=0;
				fps_d=0;
			} /* if */ 

			time+=GAME_REDRAWING_PERIOD;
			if ((act_time-time)>8*GAME_REDRAWING_PERIOD) time=act_time;

			if (game_mode==0 || game_mode==2) {

				keyboardstate[0]->cycle();
				
				{
					int old_state=state;
					if (s_action[state]==0 || 
						s_action[state]->cycle(0,0,0,active_game,0,active_game->app,&application_foreign,LOCATION_APPLICATION,keyboardstate,n_LVKs)) {
						if (state==old_state) quit=true;
					} else {
						active_game->cycle(keyboardstate,n_LVKs);
						if (terminated) quit=true;
					} /* if */ 
				}

				if (game_mode==2) {
					/* Saving replay: */ 
					/* ... */ 
				} /* if */ 
			} /* if */ 
			if (game_mode==1) {
				/* Load replay: */ 
				/* ... */ 
			} /* if */ 
			for(i=0;i<n_LVKs;i++) keyboardstate[i]->keyevents.Delete();
			tmp_strings.Delete();

			current_cycle++;
		}

		if (!quit) {
			fps_d++;
			SDL_FillRect(screen_sfc,0,0);
			if (active_game!=0) {
				active_game->draw(screen_sfc);
				sound(0,sfx_atenuation_min,sfx_atenuation_max);
				/* show FPS: */ 
/*
				{
					char tmp[80];
					FONTCONTAINER *fc;
					sprintf(tmp,"FPS: %i/%i",buffer_fps,buffer_fps_d);					
					fc=active_game->active_game->get_font("lucon.ttf",10);
					if (fc!=0 &&
						fc->ttf_font!=0) print_left(tmp,fc->ttf_font,255,255,255,screen_sfc,0,0);
				} 
*/
				/* show N COLLISIONS: */ 
/*
				{
					int i,a=0,min=-1,max=-1;
					char tmp[80];
					FONTCONTAINER *fc;

					for(i=0;i<100;i++) {
						a+=acum_collisions[i];
						if (min==-1 || acum_collisions[i]<min) min=acum_collisions[i];
						if (max==-1 || acum_collisions[i]>max) max=acum_collisions[i];
					} // for 
					sprintf(tmp,"COLLS: %i (%i [%i - %i])",n_collisions,a/100,min,max);
					fc=active_game->active_game->get_font("lucon.ttf",10);
					if (fc!=0 &&
						fc->ttf_font!=0) print_left(tmp,fc->ttf_font,255,255,255,screen_sfc,0,12);
				} 
*/
			} /* if */ 

			SDL_Flip(screen_sfc);
		} /* if */ 

	}

	if (SOUND_ENABLED) {
		Mix_HookMusic(0, 0);
		init_music(this);
	} /* if */ 

//	SDL_QuitSubSystem(SDL_INIT_VIDEO);
//	SDL_InitSubSystem(SDL_INIT_VIDEO);
	screen_sfc = SDL_SetVideoMode(SCREEN_X, SCREEN_Y, COLOUR_DEPTH, (fullscreen ? SDL_FULLSCREEN : 0));
	SDL_WM_SetCaption(application_name, 0);
	SDL_ShowCursor(SDL_DISABLE);
	SDL_EnableUNICODE(1);

} /* ActiveMOG2Application::Execute */ 


bool ActiveMOG2Application::event(SDL_Event *event)
{
	if (event->type==SDL_KEYDOWN && (game_mode==0 || game_mode==2)) {
		/* Keyboard event */ 
		if (keyboardstate!=0) {
			SDL_keysym *ks;
			ks=new SDL_keysym();
			*ks=event->key.keysym;
			keyboardstate[0]->keyevents.Add(ks);
		} /* if */ 
    } /* if */ 

	return true;
} /* ActiveMOG2Application::Event */ 


int ActiveMOG2Application::get_current_cycle(void) 
{
	return current_cycle;
} /* ActiveMOG2Application::get_current_cycle */ 


void ActiveMOG2Application::sound(VIEWPORT *vp,int sfx_atenuation_min,int sfx_atenuation_max)
{
	List<PLAYSAMPLE> todelete,l;
	PLAYSAMPLE *s,*s2;
	bool found;

	/* Filter the sounds not in the focusing windows: */ 
	if (SOUND_ENABLED) {
		l.Instance(play_samples);
		l.Rewind();
		while(l.Iterate(s)) {

			if (s->room!=0) {
				int i;
				List<VIEWPORT> l;
				VIEWPORT *v;
				List<ActiveOBJECT> l2;
				ActiveOBJECT *ao;

				found=false;

				l.Instance(active_game->viewports);
				l.Rewind();
				while(l.Iterate(v)) {
					if (v->current_interface!=0) {
						for(i=0;i<MAX_OBJECT_PLANES;i++) {
							l2.Instance(v->current_interface->objects[i]);
							l2.Rewind();
							while(l2.Iterate(ao)) {
								if (ao->object->control==CTRL_GAMEWINDOW) {
									List<ActiveOBJECT> fl;
									ActiveOBJECT *fo;
									fl.Instance(ao->gw_focusing);
									fl.Rewind();
									while(fl.Iterate(fo)) if (fo->room==s->room) found=true;
								} /* if */ 
							} /* while */ 
						} /* for */ 
					} /* if */ 
				} /* while */ 

				if (!found) todelete.Add(s);
			} /* if */ 

		} /* while */ 

		while(!todelete.EmptyP()) {
			s=todelete.ExtractIni();
			play_samples.DeleteElement(s);
			delete s;
		} /* while */ 
	} /* if */ 

	/* Filter the repeated sounds and compute the volume and pan: */ 
	if (SOUND_ENABLED) {
		int left_volume=0,right_volume=0;
		int tmp_left_volume=0,tmp_right_volume=0;
		float f,fl,fr;
		int channel;

		while(!play_samples.EmptyP()) {
			left_volume=0;
			right_volume=0;
			
			s2=play_samples[0];

			l.Instance(play_samples);
			l.Rewind();
			while(l.Iterate(s)) {
				if (s->sample==s2->sample) {				
					todelete.Add(s);

					f=s->pan/100.0F;
					fl=((1.0F-f)/2.0F);
					fr=((f+1.0F)/2.0F);
					if (fl>fr) {
						fr=fr/fl;
						fl=1.0;
					} else {
						fl=fl/fr;
						fr=1.0;
					} /* if */ 
					tmp_left_volume=int(s->volume*fl);
					tmp_right_volume=int(s->volume*fr);	
					
					if (s->placed &&
						sfx_atenuation_min>=0 && sfx_atenuation_max>=0 &&
						sfx_atenuation_min<=sfx_atenuation_max) {
						float distance=-1;

						/* compute the distance to the colsest focusing object */ 
						{
							List<ActiveOBJECT> l;
							ActiveOBJECT *po;
							float d=0;

							l.Instance(active_game->player_objects);
							l.Rewind();
							while(l.Iterate(po)) {
								if (po->room==s->room) {
									d=float(sqrt((po->x-s->x)*(po->x-s->x) + 
												 (po->y-s->y)*(po->y-s->y)));
									if (distance==-1 ||
										d<distance) distance=d;
								} /* if */ 
							} /* while */ 
						}

						if (distance<=sfx_atenuation_min) {
							if (distance==-1) {
								tmp_left_volume=0;
								tmp_right_volume=0;
							} /* if */ 
						} else {
							if (distance>=sfx_atenuation_max) {
								tmp_left_volume=0;
								tmp_right_volume=0;
							} else {
								float factor=1.0F-float(distance-sfx_atenuation_min)/float(sfx_atenuation_max-sfx_atenuation_min);
								tmp_left_volume=int(factor*tmp_left_volume);
								tmp_right_volume=int(factor*tmp_right_volume);
							} /* if */ 
						} /* if */ 
					} /* if */ 
					
					if (tmp_left_volume>left_volume) left_volume=tmp_left_volume;
					if (tmp_right_volume>right_volume) right_volume=tmp_right_volume;
				} /* if */ 
			} /* while */ 

			if (left_volume>255) left_volume=255;
			if (right_volume>255) right_volume=255;
			left_volume=(left_volume*sfx_volume)/100;
			right_volume=(right_volume*sfx_volume)/100;
			if (left_volume>255) left_volume=255;
			if (right_volume>255) right_volume=255;

			channel=Mix_PlayChannel(-1,s2->sample->sample,0);
			Mix_Volume(channel,128);
			Mix_SetPanning(channel,left_volume,right_volume);

			while(!todelete.EmptyP()) {
				s=todelete.ExtractIni();
				play_samples.DeleteElement(s);
				delete s;
			} /* while */ 

		} /* while */ 
	} /* if */ 

	play_samples.Delete();
} /* ActiveMOG2Application::sound */ 


long int ActiveMOG2Application::random(int min,int max)
{
	return random_generator->IRandom(min,max);
} /* ActiveMOG2Application::random */ 


bool ActiveMOG2Application::find_object(ActiveOBJECT *o)
{
	return active_game->find_object(o);
} /* ActiveMOG2Application::find_object */ 

