#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "auxiliar.h"
#include "MOG2_auxiliar.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BChoiceDlg.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

const int SCROLLER_WIDTH=12;
extern int SCREEN_X;
extern int SCREEN_Y;

/* Mouse state: */ 

int BWindow::old_mouse_button=0;
int BWindow::old_mouse_x=0;
int BWindow::old_mouse_y=0;
int BWindow::mouse_button=0;
int BWindow::mouse_x=0;
int BWindow::mouse_y=0;

extern int surface_flags;

/* Active window: */ 

BWindow *BWindow::active_window=0;

BWindow::BWindow(int x,int y,int dx,int dy,TTF_Font *f)
{
	font=f;

	old_x=x;
	old_y=y;
	old_dx=dx;
	old_dy=dy;
	old_stx=0;
	old_sty=0;
	old_endx=0;
	old_endy=0;
	old_vscroller_x=-1;
	old_hscroller_y=-1;
	old_vscroller_size=0;
	old_hscroller_size=0;

	local_mouse_x=0;
	local_mouse_y=0;
	mouse_button=0;

	buffer_sfc=0;
	dirty=true;
} /* BWindow::BWindow */ 



BWindow::~BWindow()
{
	if (active_window==this) active_window=0;

	if (buffer_sfc!=0) SDL_FreeSurface(buffer_sfc);
	buffer_sfc=0;
} /* BWindow::~BWindow */ 



void BWindow::draw(SDL_Surface *sfc,int x,int y,int dx,int dy)
{
	int size_x=0,size_y=0;

	old_x=x;
	old_y=y;
	old_dx=dx;
	old_dy=dy;

	contens_size(&size_x,&size_y);

	if (size_x<=dx &&
		size_y<=dy) {
		/* The subview fits in its space: */ 
		SDL_Rect s,d;

		if (buffer_sfc==0 || buffer_sfc->w!=size_x || buffer_sfc->h!=size_y) {
			if (buffer_sfc!=0) SDL_FreeSurface(buffer_sfc);
			buffer_sfc=SDL_CreateRGBSurface(SDL_SRCALPHA|surface_flags,size_x,size_y,32,0,0,0,0);
			dirty=true;
		} /* if */ 
		
		if (dirty) {
			dirty=false;
			SDL_FillRect(buffer_sfc,0,0);
			draw_contens(buffer_sfc,0,0,size_x,size_y);
		} /* if */ 

		old_stx=-1;
		old_sty=-1;
		old_endx=size_x;
		old_endy=size_y;

		s.x=0;
		s.y=0;
		s.w=size_x;
		s.h=size_y;
		d.x=x;
		d.y=y;

		SDL_BlitSurface(buffer_sfc,&s,sfc,&d);

	} else {
		/* The subview doesn't fit in its space, draw scrollers: */ 
		SDL_Rect s,d,r;
		float f1,f2;

		if (buffer_sfc==0 || buffer_sfc->w!=dx || buffer_sfc->h!=dy) {
			if (buffer_sfc!=0) SDL_FreeSurface(buffer_sfc);
			buffer_sfc=SDL_CreateRGBSurface(SDL_SRCALPHA|surface_flags,dx,dy,32,0,0,0,0);
			dirty=true;
		} /* if */ 

		if (old_stx==-1) old_stx=0;
		if (old_sty==-1) old_sty=0;
		old_endx=old_stx+dx-SCROLLER_WIDTH;
		old_endy=old_sty+dy-SCROLLER_WIDTH;

		if (dirty) {
			dirty=false;
			SDL_FillRect(buffer_sfc,0,0);
			draw_contens(buffer_sfc,old_stx,old_sty,size_x,size_y);
		} /* if */ 

		s.x=0;
		s.y=0;
		s.w=dx-SCROLLER_WIDTH;
		s.h=dy-SCROLLER_WIDTH;
		d.x=x;
		d.y=y;

		SDL_BlitSurface(buffer_sfc,&s,sfc,&d);

		/* Vertical Scroller: */ 
		old_vscroller_x=dx-SCROLLER_WIDTH;
		old_vscroller_size=dy-(SCROLLER_WIDTH+2);
		old_vscroller_rsize=size_y;
		r.x=x+dx-SCROLLER_WIDTH;
		r.y=y;
		r.w=SCROLLER_WIDTH-2;
		r.h=dy-SCROLLER_WIDTH;
		SDL_FillRect(sfc,&r,0x0000007f);

		f1=old_sty/float(size_y);
		f2=(dy-SCROLLER_WIDTH)/float(size_y);
		if (f2>=1) f2=1;
		if (f1>=1) f1=1;
		r.x=x+dx-(SCROLLER_WIDTH-1);
		r.y=1+y+int(f1*(dy-(SCROLLER_WIDTH+2)));
		r.w=SCROLLER_WIDTH-4;
		r.h=int(f2*(dy-(SCROLLER_WIDTH+2)));
		SDL_FillRect(sfc,&r,0x000000ff);

		/* Horizontal Scroller: */ 
		old_hscroller_y=dy-SCROLLER_WIDTH;
		old_hscroller_size=dx-(SCROLLER_WIDTH+2);
		old_hscroller_rsize=size_x;
		r.x=x;
		r.y=y+dy-SCROLLER_WIDTH;
		r.w=dx-SCROLLER_WIDTH;
		r.h=SCROLLER_WIDTH-2;
		SDL_FillRect(sfc,&r,0x0000007f);

		f1=old_stx/float(size_x);
		f2=(dx-SCROLLER_WIDTH)/float(size_x);
		if (f2>=1) f2=1;
		if (f1>=1) f1=1;
		r.x=1+x+int(f1*(dx-(SCROLLER_WIDTH+2)));
		r.y=y+dy-(SCROLLER_WIDTH-1);
		r.w=int(f2*(dx-(SCROLLER_WIDTH+2)));
		r.h=SCROLLER_WIDTH-4;
		SDL_FillRect(sfc,&r,0x000000ff);

	} /* if */ 

} /* BWindow::draw */ 



BButton::BButton(int x,int y,int dx,int dy,char *text,int ID)
{
	m_x=x;
	m_y=y;
	m_dx=dx;
	m_dy=dy;
	m_ID=ID;
	m_active=true;
	m_text=new char[strlen(text)+1];
	strcpy(m_text,text);
} /* BButton::BButton */ 


BButton::~BButton(void)
{
	delete []m_text;
	m_text=0;
} /* BButton::~BButton */ 


void BWindow::check_mouse(void)
{
	old_mouse_button=mouse_button;
	old_mouse_x=mouse_x;
	old_mouse_y=mouse_y;
	SDL_PumpEvents();
	mouse_button=SDL_GetMouseState(&mouse_x, &mouse_y);
} /* BWindow::check_mouse */ 


void BWindow::local_draw_button(SDL_Surface *sfc,BButton *b,int stx,int sty,bool selected)
{
	draw_button(sfc,b->m_x-stx,b->m_y-sty,b->m_dx,b->m_dy,b->m_text,font,selected);
} /* BWindow::local_draw_button */ 


void BWindow::set_active_button(int ID)
{
	BButton *b;
	List<BButton> l;
	
	l.Instance(buttons);
	l.Rewind();
	while(l.Iterate(b)) {
		if (b->m_ID==ID) b->set_active();
	} /* while */ 
} /* BWindow::set_active_button */ 


void BWindow::set_inactive_button(int ID)
{
	BButton *b;
	List<BButton> l;
	
	l.Instance(buttons);
	l.Rewind();
	while(l.Iterate(b)) {
		if (b->m_ID==ID) b->set_inactive();
	} /* while */ 
} /* BWindow::set_inactive_button */ 


bool BWindow::is_dirty(void)
{
	return dirty;
} /* BWindow::is_dirty */ 


void BWindow::set_dirty(void)
{
	dirty=true;
} /* BWindow::set_dirty */ 


void BWindow::new_window_coordinates(int dx,int dy,int *x,int *y)
{
	*x=mouse_x-dx/2;
	*y=mouse_y;
	if (*x<0) *x=0;
	if (*y<0) *y=0;
	if (*x+dx>SCREEN_X) *x=SCREEN_X-dx;
	if (*y+dy>SCREEN_Y) *y=SCREEN_Y-dy;
} /* BWindow::new_window_coordinates */ 



void BWindow::draw_message(char *message) 
{
	int x,y;

	new_window_coordinates(get_text_width(message,font)+40,50,&x,&y);
	BChoiceDlg *dlg2=new BChoiceDlg(x,y,get_text_width(message,font)+40,50,message,"Accept",0,font);
	dlg2->Execute();
	delete dlg2;
} /* BWindow::draw_message */ 


/* -- Functions to redefine for each new window: */ 

void BWindow::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	BButton *b;

	buttons.Rewind();
	while(buttons.Iterate(b)) {
		if (b->m_active) {
			if (local_mouse_x>=old_stx &&
				local_mouse_y>=old_sty &&
				local_mouse_x<=old_endx &&
				local_mouse_y<=old_endy && 
				local_mouse_x>=b->m_x && local_mouse_x<b->m_x+b->m_dx &&
				local_mouse_y>=b->m_y && local_mouse_y<b->m_y+b->m_dy) {
				local_draw_button(sfc,b,stx,sty,true);
			} else {
				local_draw_button(sfc,b,stx,sty,false);
			} /* if */ 
		} /* if */ 
	} /* while */ 

} /* BWindow::draw_contens */ 


bool BWindow::cycle(int stx,int sty)
{ 
	BButton *b;

	if (old_stx+(old_hscroller_size+2)>old_hscroller_rsize) old_stx=old_hscroller_rsize-(old_hscroller_size+2);
	if (old_stx<0) old_stx=0;
	if (old_sty+(old_vscroller_size+2)>old_vscroller_rsize) old_sty=old_vscroller_rsize-(old_vscroller_size+2);
	if (old_sty<0) old_sty=0;

	local_mouse_x=(mouse_x-stx)+old_stx;
	local_mouse_y=(mouse_y-sty)+old_sty;

	if (mouse_x-stx>=0 &&
		mouse_y-sty>=0 &&
		mouse_x-stx<old_dx &&
		mouse_y-sty<old_dy) {

		if (mouse_button&SDL_BUTTON(1)) {
			/* LEFT BUTTON PRESSED: */ 
			active_window=this;

			if (old_vscroller_x!=-1 &&
				(local_mouse_x-old_stx)>=old_vscroller_x &&
				(local_mouse_x-old_stx)<=old_vscroller_x+SCROLLER_WIDTH &&
				(local_mouse_y-old_sty)>=0 && 
				(local_mouse_y-old_sty)<old_vscroller_size) {
				/* VSCROLLER CLICK: */ 
				int dy;

				dy=mouse_y-old_mouse_y;
				dy=int(dy*(old_vscroller_rsize/float(old_vscroller_size)));

				old_sty+=dy;
				if (old_sty+(old_vscroller_size+2)>old_vscroller_rsize) old_sty=old_vscroller_rsize-(old_vscroller_size+2);
				if (old_sty<0) old_sty=0;
				dirty=true;
			} /* if */ 

			if (old_hscroller_y!=-1 &&
				(local_mouse_y-old_sty)>=old_hscroller_y &&
				(local_mouse_y-old_sty)<=old_hscroller_y+SCROLLER_WIDTH &&
				(local_mouse_x-old_stx)>=0 && 
				(local_mouse_x-old_stx)<old_hscroller_size) {
				/* HSCROLLER CLICK: */ 
				int dx;

				dx=mouse_x-old_mouse_x;
				dx=int(dx*(old_hscroller_rsize/float(old_hscroller_size)));

				old_stx+=dx;
				if (old_stx+(old_hscroller_size+2)>old_hscroller_rsize) old_stx=old_hscroller_rsize-(old_hscroller_size+2);
				if (old_stx<0) old_stx=0;
				dirty=true;
			} /* if */ 
		} /* if */ 

		/* CHECK FOR BUTTONS: */ 
		buttons.Rewind();
		while(buttons.Iterate(b)) {

			if (b->m_active) {
				if (local_mouse_x>=old_stx &&
					local_mouse_y>=old_sty &&
					local_mouse_x<=old_endx &&
					local_mouse_y<=old_endy && 
					local_mouse_x>=b->m_x && local_mouse_x<b->m_x+b->m_dx &&
					local_mouse_y>=b->m_y && local_mouse_y<b->m_y+b->m_dy) {
					if (!b->mouse_over) {
						b->mouse_over=true;
						dirty=true;
					} /* if */ 
					if (mouse_button&SDL_BUTTON(1) &&
						!(old_mouse_button&SDL_BUTTON(1))) {

						/* Reset mouse status: */ 						
						old_mouse_button=mouse_button;
						if (!button_pressed(b->m_ID)) return false;
						old_mouse_button=mouse_button;
					} /* if */ 

				} else {
					if (b->mouse_over) {
						b->mouse_over=false;
						dirty=true;
					} /* if */ 
				} /* if */ 
			} /* if */ 
		} /* while */ 

	} /* if */ 

	return true;
} /* BWindow::cycle */ 



void BWindow::contens_size(int *dx,int *dy)
{
	BButton *b;

	*dx=1;
	*dy=1;

	buttons.Rewind();
	while(buttons.Iterate(b)) {
		if (b->m_active) {
			if (b->m_x+b->m_dx>*dx) *dx=b->m_x+b->m_dx;
			if (b->m_y+b->m_dy>*dy) *dy=b->m_y+b->m_dy;
		} /* if */ 
	} /* while */ 
} /* BWindow::contens_size */ 



bool BWindow::button_pressed(int)
{
	return true;
} /* BWindow::button_pressed */ 



bool BWindow::event(SDL_Event *)
{
	return true;
} /* BWindow::Event */ 
