#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_net.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"
#include "mog2multiplayer.h"
#include "interfaces.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "BListWnd.h"
#include "BMultiListWnd.h"

#include "RToolsWnd.h"

#include "GamePReviewDlg.h"
#include "RMoveDlg.h"
#include "BPlaneSelectionDlg.h"
#include "ParallaxWnd.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

RToolsWnd::RToolsWnd(int x,int y,int dx,int dy,EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font) 
{
	BButton *b;
	editor=e;

	b=new BButton(100,4,50,14,"ADD",1);
	buttons.Add(b);

	b=new BButton(100,24,50,14,"SUB",2);
	buttons.Add(b);

	b=new BButton(90,44,70,14,"OPTIONS",11);
	buttons.Add(b);

	b=new BButton(4,74,80,14,"EDITING",3);
	buttons.Add(b);

	b=new BButton(4,94,80,14,"MODE",4);
	buttons.Add(b);

	b=new BButton(4,114,80,14,"GRID",5);
	buttons.Add(b);

	b=new BButton(4,140,60,14,"CLEAR",6);
	buttons.Add(b);

	b=new BButton(74,140,60,14,"COPY",7);
	buttons.Add(b);

	b=new BButton(144,140,60,14,"PASTE",8);
	buttons.Add(b);

	b=new BButton(144,94,60,34,"PREVIEW",9);
	buttons.Add(b);

	b=new BButton(155,28,50,14,"MOVE",10);
	buttons.Add(b);

	lw=new BMultiListWnd(4,4,80,60,&plane_names,font);

} /* RToolsWnd::RToolsWnd */ 


RToolsWnd::~RToolsWnd()
{
	delete lw;
} /* RToolsWnd::~RToolsWnd */ 


bool RToolsWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (editor->room!=0) {

		while(plane_names.Length()>editor->room->used_object_planes) {
			lw->sub_element(plane_names.Length()-1);
			lw->set_dirty();
		} /* while */ 
		while(plane_names.Length()<editor->room->used_object_planes) {
			char tmp[80],*tmp2;
	
			sprintf(tmp,"Plane %i",plane_names.Length());
			tmp2=new char[strlen(tmp)+1];
			strcpy(tmp2,tmp);
			lw->add_element_name(tmp2);
			lw->set_selected_name(tmp2);
			
			lw->set_dirty();
		} /* while */ 

		lw->cycle(stx+4,sty+4);
		if (lw->is_dirty()) editor->set_all_dirty();

		if ((mouse_button&SDL_BUTTON(1) &&
		    !(old_mouse_button&SDL_BUTTON(1))) ||
			(mouse_button&SDL_BUTTON(3) &&
		    !(old_mouse_button&SDL_BUTTON(3)))) {

			if (local_mouse_x>=160 &&
				local_mouse_y>=4 &&
				local_mouse_y<4+TTF_FontHeight(font) &&
				local_mouse_x<160+get_text_width("X: 999999",font)) {
				/* Change DX: */ 
				BIntegerEdDlg *dlg;
				int y,x;

				new_window_coordinates(140,60,&x,&y);

				dlg=new BIntegerEdDlg(x,y,140,60,"Enter room width:",&(editor->room->dx),4,font);
				dlg->Execute();
				delete dlg;
				editor->set_all_dirty();
			} /* if */ 

			if (local_mouse_x>=160 &&
				local_mouse_y>=14 &&
				local_mouse_y<14+TTF_FontHeight(font) &&
				local_mouse_x<160+get_text_width("X: 999999",font)) {
				/* Change DX: */ 
				BIntegerEdDlg *dlg;
				int y,x;

				new_window_coordinates(140,60,&x,&y);

				dlg=new BIntegerEdDlg(x,y,140,60,"Enter room height:",&(editor->room->dy),4,font);
				dlg->Execute();
				delete dlg;
				editor->set_all_dirty();
			} /* if */ 
		} /* if */ 
	} /* if */ 

	return true;
} /* REdiRToolsWndtWnd::cycle */ 


void RToolsWnd::contens_size(int *dx,int *dy)
{
	int max_x=0,max_y=0;

	BWindow::contens_size(dx,dy);

	if (editor->room!=0) {
		max_x=160+get_text_width("DX: 0000",font);
	} /* if */ 

	if (max_x>*dx) *dx=max_x;
	if (max_y>*dy) *dy=max_y;
} /* RToolsWnd::contens_size */ 


void RToolsWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	lw->draw(sfc,4-stx,4-sty,80,60);
	if (editor->rv_editmode==1) print_left("ERASE",font,255,255,255,sfc,92-stx,98-sty);
	if (editor->rv_editmode==2) print_left("REGION",font,255,255,255,sfc,92-stx,98-sty);
	if (editor->rv_editmode==3) print_left("MODIFY",font,255,255,255,sfc,92-stx,98-sty);
	if (editor->rv_editmode==4) print_left("ABSCOORD",font,255,255,255,sfc,92-stx,98-sty);
	if (editor->rv_editmode==5) print_left("DRAG",font,255,255,255,sfc,92-stx,98-sty);

	if (editor->rv_grid==0) print_left("1x1",font,255,255,255,sfc,92-stx,118-sty);
	if (editor->rv_grid==1) print_left("2x2",font,255,255,255,sfc,92-stx,118-sty);
	if (editor->rv_grid==2) print_left("4x4",font,255,255,255,sfc,92-stx,118-sty);
	if (editor->rv_grid==3) print_left("8x8",font,255,255,255,sfc,92-stx,118-sty);
	if (editor->rv_grid==4) print_left("16x16",font,255,255,255,sfc,92-stx,118-sty);
	if (editor->rv_grid==5) print_left("32x32",font,255,255,255,sfc,92-stx,118-sty);

	if (editor->room!=0) {
		char tmp[80];

		draw_rectangle(sfc,2-stx,2-sty,83,63,SDL_MapRGB(sfc->format,255,255,255));

		sprintf(tmp,"DX: %i",editor->room->dx);
		print_left(tmp,font,255,255,255,sfc,160-stx,4-sty);
		sprintf(tmp,"DY: %i",editor->room->dy);
		print_left(tmp,font,255,255,255,sfc,160-stx,14-sty);

		sprintf(tmp,"PLANE %i",editor->rv_editing);
		print_left(tmp,font,255,255,255,sfc,92-stx,78-sty);
	} /* if */ 

} /* RToolsWnd::draw_contens */ 


bool RToolsWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		/* Add a plane: */ 
		editor->room->used_object_planes++;
		if (editor->room->used_object_planes>MAX_OBJECT_PLANES) editor->room->used_object_planes=MAX_OBJECT_PLANES;
		editor->set_all_dirty();
		break;

	case 2:
		/* Sub a plane: */ 
		if (editor->room->used_object_planes>0) {
			editor->room->used_object_planes--;
			editor->room->objects[editor->room->used_object_planes].Delete();
			editor->room->tiles[editor->room->used_object_planes].Delete();
		} /* if */ 
		editor->set_all_dirty();
		break;

	case 3:
		editor->rv_editing++;
		if (editor->rv_editing>=editor->room->used_object_planes) editor->rv_editing=0;
		editor->set_all_dirty();
		break;

	case 4:
		editor->rv_editmode++;
		if (editor->rv_editmode>5) editor->rv_editmode=1;
		editor->set_all_dirty();
		break;

	case 5:
		editor->rv_grid++;
		if (editor->rv_grid>=6) editor->rv_grid=0;
		editor->set_all_dirty();
		break;

	case 6:
		/* CLEAR: */ 
		{
			BPlaneSelectionDlg *dlg;
			int y,x;
			int i;

			new_window_coordinates(140,100,&x,&y);
			dlg=new BPlaneSelectionDlg(x,y,140,100,editor->room->used_object_planes,"Clear",font);
			dlg->Execute();
		
			for(i=0;i<editor->room->used_object_planes;i++) {
				if (dlg->selected_p(i)) {
					editor->room->objects[i].Delete();
					editor->room->tiles[i].Delete();
				} /* if */ 
			} /* for */ 
			delete dlg;
			editor->set_all_dirty();
		}
		break;

	case 7:
		/* COPY: */ 
		{
			BPlaneSelectionDlg *dlg;
			int res=0;
			int y,x;
			int i;

			new_window_coordinates(140,100,&x,&y);
			dlg=new BPlaneSelectionDlg(x,y,140,100,editor->room->used_object_planes,"Copy",font);
			dlg->Execute();

			if (res!=0) {
				int i;
				
				for(i=0;i<MAX_OBJECT_PLANES;i++) {
				    editor->rv_cliproom.objects[i].Delete();
				    editor->rv_cliproom.tiles[i].Delete();
				} /* for */ 
			} /* if */ 

			for(i=0;i<editor->room->used_object_planes;i++) {
				if (dlg->selected_p(i)) {
					editor->rv_cliproom.objects[i].Copy(editor->room->objects[i]);
				    editor->rv_cliproom.tiles[i].Copy(editor->room->tiles[i]);
				} else {
					editor->rv_cliproom.objects[i].Delete();
					editor->rv_cliproom.tiles[i].Delete();
				} /* if */ 
			} /* for */ 
			delete dlg;
		}
		break;

	case 8:
		/* PASTE: */ 
		{
			BVChoiceDlg *dlg;
			int res=0;

			dlg=new BVChoiceDlg(-1,-1,0,0,"Choose action:","Paste over\nPaste replacing\n",&res,font);
			dlg->Execute();
			delete dlg;

			if (res!=0) {
				if (res==2) {
					int i;
					
					for(i=0;i<MAX_OBJECT_PLANES;i++) {
					    editor->room->objects[i].Delete();
					    editor->room->tiles[i].Delete();
					} /* for */ 
				} /* if */ 

				dlg=new BVChoiceDlg(-1,-1,0,0,"Choose action:","Paste single\nPaste pattern\n",&res,font);
				dlg->Execute();
				delete dlg;

				if (res==1) {
					int i;
					
					for(i=0;i<MAX_OBJECT_PLANES;i++) {
					    editor->room->objects[i].Append(editor->rv_cliproom.objects[i]);
					    editor->room->tiles[i].Append(editor->rv_cliproom.tiles[i]);
					} /* for */ 
				} /* if */ 

				if (res==2) {
					int i;
					long int stx=0,sty=0;
					long int edx=0,edy=0;
					bool first=true;
					PLACED_TILE *t;
					PLACED_OBJECT *o;

					/* Compute de bounding box of the pattern: */ 
					for(i=0;i<MAX_OBJECT_PLANES;i++) {
					    editor->rv_cliproom.tiles[i].Rewind();
					    while(editor->rv_cliproom.tiles[i].Iterate(t)) {
						if (first) {
							stx=t->x;
							sty=t->y;
							edx=t->x+t->tile->r.w;
							edy=t->y+t->tile->r.h;
							first=false;
						} else {
							if (t->x<stx) stx=t->x;
							if (t->y<sty) sty=t->y;
							if (t->x+t->tile->r.w>edx) edx=t->x+t->tile->r.w;
							if (t->y+t->tile->r.h>edy) edy=t->y+t->tile->r.h;
						} /* if */ 
					    } /* while */ 
					    editor->rv_cliproom.objects[i].Rewind();
					    while(editor->rv_cliproom.objects[i].Iterate(o)) {
						if (o->object->ntiles>0) {
							if (first) {
								stx=o->x;
								sty=o->y;
								edx=o->x+o->object->tiles[0]->r.w;
								edy=o->y+o->object->tiles[0]->r.h;
								first=false;
							} else {
								if (o->x<stx) stx=o->x;
								if (o->y<sty) sty=o->y;
								if (o->x+o->object->tiles[0]->r.w>edx) edx=o->x+o->object->tiles[0]->r.w;
								if (o->y+o->object->tiles[0]->r.h>edy) edy=o->y+o->object->tiles[0]->r.h;
							} /* if */ 
						} /* if */ 
					    } /* while */ 
					} /* for */ 
					
					edx=edx-stx;
					edy=edy-sty;

					if (edx>0 && edy>0) {
						int i;
						long int x,y;
						PLACED_TILE *t2;
						PLACED_OBJECT *o2;

						/* Repeat te pattern: */ 
						for(x=0;x<editor->room->dx*TILESIZE;x+=edx) {
							for(y=0;y<editor->room->dy*TILESIZE;y+=edy) {
							    for(i=0;i<MAX_OBJECT_PLANES;i++) {
								editor->rv_cliproom.tiles[i].Rewind();
								while(editor->rv_cliproom.tiles[i].Iterate(t)) {
									t2=new PLACED_TILE(*t);
									t2->x=(t2->x-stx)+x;
									t2->y=(t2->y-sty)+y;
									if (t2->x<editor->room->dx*TILESIZE &&
										t2->y<editor->room->dy*TILESIZE) editor->room->tiles[i].Add(t2);
													    else delete t2;
								} /* while */ 
								editor->rv_cliproom.objects[i].Rewind();
								while(editor->rv_cliproom.objects[i].Iterate(o)) {
									o2=new PLACED_OBJECT(*o);
									o2->x=(o2->x-stx)+x;
									o2->y=(o2->y-sty)+y;
									if (o2->x<editor->room->dx*TILESIZE &&
										o2->y<editor->room->dy*TILESIZE) editor->room->objects[i].Add(o2);
													    else delete o2;
								} /* while */ 
							    } /* for */ 
		    
							} /* for */ 
						} /* for */ 
					
					} /* if */ 		
				} /* if */ 
			} /* if */ 
			editor->set_all_dirty();
		}
		break;

	case 9:
		/* ROOM PREVIEW: */ 
		{
			BGamePreviewDlg *dlg=new BGamePreviewDlg(40,40,300,300,editor->game,editor->map,editor->room,font);
			dlg->Execute();
			delete dlg;
		}
		break;

	case 10:
		/* MOVE TiLES/OBJECTS: */ 
		{
			BRMoveDlg *dlg;

			dlg=new BRMoveDlg(40,40,200,100,editor->game,editor->map,editor->room,font);
			dlg->Execute();
			delete dlg;

			editor->set_all_dirty();
		}
		break;

	case 11:
		/* PARALLAX options: */ 
		{
			int x,y;
			BParallaxWnd *dlg;

			new_window_coordinates(260,170,&x,&y);

			dlg=new BParallaxWnd(x,y,260,170,editor->room,font);
			dlg->Execute();
			delete dlg;

			editor->set_all_dirty();
		}
		break;

	} /* switch */ 

	return false;
} /* RToolsWnd::button_pressed */ 
